using System;
using System.Data;
using System.Xml;
using System.IO;
using angGoGo.GalleryNET.GalleryData;
using angGoGo.GalleryNET.SystemFramework;

namespace angGoGo.GalleryNET.Core
{
	/// <summary>
	/// Stores the album information for a specified directory
	/// </summary>
	internal class AlbumInfo:IDisposable
	{
		// the physical album.xml location
		private string dbname = "";
		// dataset to operate
		private albumLogData dsInfo;

		/// <summary>
		/// Initialize a new instance of AlbumInfo
		/// </summary>
		public AlbumInfo()
		{
			// assign the dbname
			dbname = ApplicationConfiguration.SettingFolder + ApplicationConfiguration.ClickLogFile;

			dsInfo = new albumLogData();

			// if album.xml doesnot exist, create it
			// otherwise read from it
			if (!File.Exists(dbname))
				initDB();
			else
			{

				dsInfo.ReadXml(dbname);
			}
		}

		/// <summary>
		/// Release the resource of this object
		/// </summary>
		public void Dispose()
		{
			if (dsInfo!=null) dsInfo.Dispose();
		}

		/// <summary>
		/// Gets the click count of the specialized file
		/// </summary>
		/// <param name="filename">The specialized file name</param>
		/// <returns>the click count</returns>
		public int GetClick(string filename)
		{
			// select from the dataset
			albumLogData.LogRow[] lr = (albumLogData.LogRow[])dsInfo.Log.Select("FileName='" + filename + "'");

			// if there is result, return the record. 
			if (lr.Length>0)
			{
				return Convert.ToInt32( lr[0].Click );
			}
			else 
			{
				return 0;
			}
		}

		/// <summary>
		/// Increase the specialized file click count
		/// </summary>
		/// <param name="filename">The specialized file name, recommend to use virtual file name</param>
		/// <returns>Always return successful value</returns>
		public bool AddClick(string filename)
		{
			albumLogData.LogRow[] lr = (albumLogData.LogRow[])dsInfo.Log.Select("FileName='" + filename + "'");

			// if there is no such a record, create it
			if (lr.Length == 0)
			{
				albumLogData.LogRow newLog = dsInfo.Log.NewLogRow();
				newLog.Click = 1;
				newLog.FileName = filename;
				dsInfo.Log.AddLogRow(newLog);
			}
			else // otherwise increase the click field
			{
				lr[0].Click = lr[0].Click + 1;
			}

			// this function always return true
			return true;
		}

		protected void initDB()
		{
			dsInfo.Namespace = "angGoGo.GalleryNET";
			dsInfo.DataSetName = "AlbumInfo";
		}
		
		/// <summary>
		/// Save the modified album.xml
		/// </summary>
		/// <returns>Return true if succeed</returns>
		public bool Save()
		{
			try 
			{
				dsInfo.WriteXml(dbname);
				return true;
			}
			catch (Exception ex)
			{
				throw new GalleryNETException(ex.Message);
			}
		}

		/// <summary>
		/// Abandon all modification in this ablum.xml
		/// </summary>
		/// <returns>Return true if succeed</returns>
		public bool Reset()
		{
			try 
			{
				dsInfo.Reset();
				return true;
			}
			catch 
			{
				return false;
			}
		}

		/// <summary>
		/// Reset all click count
		/// </summary>
		/// <returns>If reset procedure succeed, returns true</returns>
		public bool ResetCount()
		{
			try 
			{
				albumLogData.LogRow[] lr = (albumLogData.LogRow[])dsInfo.Log.Select();
				for (int i=0; i< lr.Length; i++)
				{
					lr[i].Click = 0;
				}
				return true;
			}
			catch 
			{
				return false;
			}
		}

		/// <summary>
		/// Reset the specified file's click count
		/// </summary>
		/// <param name="filename">The specified file name</param>
		/// <returns>If reset procedure succeed, returns true</returns>
		public bool ResetCount(string filename)
		{
			try 
			{
				albumLogData.LogRow[] lr = (albumLogData.LogRow[])dsInfo.Log.Select("FileName='" + filename + "'");
				for (int i=0; i< lr.Length; i++)
				{
					lr[i].Click = 0;
				}
				return true;
			}
			catch 
			{
				return false;
			}
		}

		/// <summary>
		///		Gets the top click count pictures. Reserved, return empty in current version
		///		<remarks>
		///			So far in the log database, I store the click count and virtual file name.
		///			
		///			But, how can I get the physcial file name for each of them so that I can
		///			setup the GalleryInfo object.
		///			
		///			I need to have a function to do the following.
		///			1. Delete VirutalPath from the stored virtual file name to get a string A
		///			2. Connect string A with PhysicalPath to get this file's physical full path
		///			3. Use the full path to initialize the GalleryInfo
		///			
		///			On the other hand, I should write one more function to find out the addonpath 
		///			for a virutal filename.
		///		</remarks>
		/// </summary>
		/// <param name="count">Total number</param>
		/// <returns>Nothing</returns>
		public GalleryInfo[] GetRank(int count)
		{
			GalleryInfo[] gis = new GalleryInfo[count];
			return gis;
		}
	}

}
