using System;
using System.Drawing;
using System.Drawing.Imaging;

namespace angGoGo.GalleryNET.Core
{
	/// <summary>
	/// A helper class for bitmap controll
	/// </summary>
	internal class BitmapHelper
	{
		private static ImageCodecInfo GetEncoderInfo(String mimeType)
		{
			int j;
			ImageCodecInfo[] encoders;
			encoders = ImageCodecInfo.GetImageEncoders();
			for(j = 0; j < encoders.Length; ++j)
			{
				if(encoders[j].MimeType == mimeType)
					return encoders[j];
			}
			return null;
		}

		/// <summary>
		/// Gets the thumbnail image from specialized width and height
		/// </summary>
		/// <param name="bitmap">The source image</param>
		/// <param name="width">The width of thumbnail image</param>
		/// <param name="height">The height of thumbnail image</param>
		/// <returns>The thumbnail image</returns>
		public static Image GetThumbnail(Bitmap bitmap,int width,int height)
		{
			Image myThumbnail = bitmap.GetThumbnailImage(
				width, height, null, IntPtr.Zero);
			return myThumbnail;
		}

		/// <summary>
		/// Gets the size scaled in specialized width and height
		/// </summary>
		/// <param name="bitmap">The source image</param>
		/// <param name="width">The scaling width</param>
		/// <param name="height">The scaling height</param>
		/// <returns>The scaled size</returns>
		public static SizeF ScaleSize(Bitmap bitmap,int width,int height)
		{
			System.Drawing.Rectangle rect = new Rectangle(0,0,width,height);
			SizeF sizef=new SizeF(bitmap.Width / bitmap.HorizontalResolution,
				bitmap.Height / bitmap.VerticalResolution);
			float fScale = Math.Min(rect.Width / sizef.Width,rect.Height / sizef.Height);
			sizef.Width *= fScale;
			sizef.Height *= fScale;

			return sizef;
		}

		/// <summary>
		/// Save the image to a jpeg formation file
		/// </summary>
		/// <param name="bitmap">The source image</param>
		/// <param name="filename">The destination jpeg file name</param>
		/// <param name="quality">The quality to save jpeg</param>
		/// <returns>Always return true if saving is successful</returns>
		/// <remarks>This code comes from MSDN, please refer to the corresponding subject</remarks>
		public static bool SaveJpeg(Bitmap bitmap,string filename,int quality)
		{
			ImageCodecInfo myImageCodecInfo;
			Encoder myEncoder;
			EncoderParameter myEncoderParameter;
			EncoderParameters myEncoderParameters;

			myImageCodecInfo = GetEncoderInfo("image/jpeg");
			myEncoder = Encoder.Quality;
			myEncoderParameters = new EncoderParameters(1);
			myEncoderParameter = new EncoderParameter(myEncoder,quality);
			myEncoderParameters.Param[0] = myEncoderParameter;

			bitmap.Save(filename,myImageCodecInfo,myEncoderParameters);
			return true;
		}
	}
}
