
namespace angGoGo.GalleryNET.Core
{
	using System;
	using System.Web;
	using System.IO;
	using System.Drawing;
	using angGoGo.GalleryNET.SystemFramework;
	using angGoGo.GalleryNET.GalleryData;
	using angGoGo.GalleryNET.Template;

	/// <summary>
	///	To show a specified image on the webpage according to TemplateShowFile
	/// </summary>
	public class GalleryImage
	{
		private HttpResponse response;

		/// <summary>
		/// Initializes a new instance of GalleryImage
		/// </summary>
		/// <param name="resp">HttpResponse object is used to write the content in template.</param>
		public GalleryImage(HttpResponse resp)
		{
			response = resp;
		}

		/// <summary>
		/// Show the specified file on the webpage
		/// </summary>
		/// <param name="filename">The specified file name</param>
		/// <param name="addonPath">
		///		The addonPath is specified as the rest part after [Album] directory
		///		<remark>
		///		For example, the base album folder name is [Album]. There is 
		///		a folder named [Girl] under the base. And, under the [Girl] folder there are
		///		three picture files; they are 1.jpg, 2.jpg, 3.jpg. To show 2.jpg on webpage,
		///		pass two parameters into this function, one is "2.jpg", specifies the filename
		///		to display; another is "Girl", specifies the addonPath.
		///		
		///		Regularly, the picture file name should be 
		///		[Base Folder Name]\[Sub Folder Name1]\[Sub Folder Name2]\[...]\FileName
		///		
		///		the addonPath is the section that deletes "[Base Folder Name]\" and "\FileName"
		///		</remark>
		/// </param>
		public void Show(string filename, string addonPath)
		{
			// get this specified file's physical location
			string physicalFileName = ApplicationConfiguration.PhysicalPath + addonPath + "\\" + filename;

			// if the file doesn't exist, throw the exception
			if (!File.Exists(physicalFileName)) throw new GalleryNETException("Cannot find " + physicalFileName);;

			// create a FileInfo according to the specified file
			FileInfo fileInfo = new FileInfo(physicalFileName);
			// create a new GalleryInfo object to store the information
			GalleryInfo gi = new GalleryData.GalleryInfo();

			gi.FileName = fileInfo.Name;
			gi.FullName = fileInfo.FullName;
			gi.Size = fileInfo.Length;
			gi.VirtualFileName = ApplicationConfiguration.VirtualPath + addonPath + "/" + filename;
			gi.CreateTime = fileInfo.CreationTime;
			
			Bitmap tempBitmap = new Bitmap(physicalFileName);
			SizeF sizef = BitmapHelper.ScaleSize(tempBitmap,ApplicationConfiguration.ThumbWidth,ApplicationConfiguration.ThumbHeight);

			gi.PixelFormat = tempBitmap.PixelFormat.ToString();
			gi.Width = tempBitmap.Width;
			gi.Height = tempBitmap.Height; 
			gi.ThumbName = fileInfo.DirectoryName + "/_thumbnails/_" + fileInfo.Name;
			gi.ThumbHeight = (int)sizef.Width;
			gi.ThumbWidth = (int)sizef.Height;

			AlbumInfo ai = new AlbumInfo();
			// use VirutalFileName as the identity, because FileName might be duplicated, but with
			// a virtual folder name it must be unique.
			gi.ClickCount = ai.GetClick(gi.VirtualFileName);

			// let angGoGo.GalleryNET.Template.Builder to do the show job
            Template.Builder.Translate(ApplicationConfiguration.TemplateShowFile,gi,addonPath,response);	
		
			// Detect the configuration, if it needs logging the click count, then log it
			if (ApplicationConfiguration.ClickEnabled) 
			{
				ai.AddClick(gi.VirtualFileName);
				ai.Save();
			}

			// release AlbumInfo object
			ai.Dispose();
		}
	}
}
