using System;
using System.IO;
using angGoGo.GalleryNET.SystemFramework;
using angGoGo.GalleryNET.GalleryData;
using System.Web;

namespace angGoGo.GalleryNET.Template
{
	/// <summary>
	///		Builder class provides static methods to translate the content in template file 
	///		into HTML code.
	///		
	///		<remarks>
	///			All HTML output job should be assigned to Template.
	///			
	///			But, regularly, this class won't be called directly in ASP.NET file but by 
	///			angGoGo.GalleryNET.Core.GalleryGrid and GalleryImage class.
	///		</remarks>
	/// </summary>
	public class Builder
	{
		//
		//  tag statement for showing album
		//
		private const string TAG_FILENAME = "<!--filename-->";
		private const string TAG_FULLNAME = "<!--fullname-->";
		private const string TAG_VIRTURALFILENAME  = "<!--virtualfilename-->";
		private const string TAG_THUMBNAME = "<!--thumbname-->";
		private const string TAG_PIXELFORMAT = "<!--pixelformat-->";
		private const string TAG_SIZE = "<!--size-->";
		private const string TAG_WIDTH = "<!--width-->";
		private const string TAG_HEIGHT = "<!--height-->";
		private const string TAG_CLICKCOUNT = "<!--clickcount-->";
		private const string TAG_CREATETIME = "<!--createtime-->";

		//
		//  tag statement for showing folders
		//
		private const string TAG_FOLDERNAME = "<!--foldername-->";
		private const string TAG_UPDATETIME = "<!--updatetime-->";
		private const string TAG_UPDATEMARK = "<!--updatemark-->";
		//
		//  pending, these two are useless in this version
		//
		private const string TAG_INDEXIMAGE = "<!--indeximage-->";
		private const string TAG_DESCRIPTION = "<!--description-->";

		//
		//  tag statement for both
		//
		private const string TAG_ADDONPATH = "<!--addonpath-->";

		/*
		static Builder()
		{
			//
			// so far I don't think I need anything in this consturctor
			//
		}
		*/

		#region Translate DirectoryInfo according to different template

		/// <summary>
		/// Translate DirectoryInfo according to different template and output to HTML directly
		/// </summary>
		/// <param name="templateFile">The template file with full path</param>
		/// <param name="directoryInfo">The specified DirectoryInfo object</param>
		/// <param name="addonpath">The specified addon path</param>
		/// <param name="resp">HttpResponse object used to output</param>
		public static void Translate(string templateFile, DirectoryInfo directoryInfo, 
			string addonpath, HttpResponse resp)
		{
			string output = "";
			StreamReader streamReader = new StreamReader(templateFile);

			// read content in the specified template file
			try 
			{
				output = streamReader.ReadToEnd();
				streamReader.Close();
			}
			catch
			{
				throw new Exception("Cannot read template file " + templateFile);
			}

			if (output == "") return;

			// just in case, change the template content to all lower letters
			output = output.ToLower();

			output = output.Replace(TAG_FOLDERNAME,directoryInfo.Name);
			output = output.Replace(TAG_FULLNAME, directoryInfo.FullName);
			output = output.Replace(TAG_UPDATETIME, directoryInfo.LastWriteTime.ToShortDateString());

			System.TimeSpan timeSpan = DateTime.Today.Subtract(directoryInfo.LastWriteTime);
			// if last update to this directory in within 7 days, 
			// replace tag "<updatemark>" to "Updated"
			if (timeSpan.Days <= 7)
			{
				output = output.Replace(TAG_UPDATEMARK, "Updated");
			}
			else
			{
				output = output.Replace(TAG_UPDATEMARK, "");
			}

			// change here for V2.1
			// to support unlimited sub-folders
			if (addonpath!="") 
			{
				output = output.Replace(TAG_ADDONPATH,addonpath + "/");
			}
			else
			{
				output = output.Replace(TAG_ADDONPATH, "");
			}

			resp.Write(output);
		}

		/// <summary>
		/// Translate DirectoryInfo objects according to different template and output to HTML directly
		/// </summary>
		/// <param name="templateFile">The template file with full path</param>
		/// <param name="directoryInfos">The specified DirectoryInfo objects</param>
		/// <param name="addonpath">The specified addon path</param>
		/// <param name="resp">HttpResponse object used to output</param>
		public static void Translate(string templateFile, DirectoryInfo[] directoryInfos, string addonpath, HttpResponse resp)
		{
			string templateContent = "";

			StreamReader streamReader = new StreamReader(templateFile);

			try 
			{
				templateContent = streamReader.ReadToEnd();
				streamReader.Close();
			}
			catch
			{
				throw new Exception("Cannot read template file " + templateFile);
			}

			if (templateContent == "") return;

			// just in case, change the template content to all lower letters
			templateContent = templateContent.ToLower();
			string output = templateContent;

			for (int i=0;i<directoryInfos.Length;i++)
			{
				output = output.Replace(TAG_FOLDERNAME,directoryInfos[i].Name);
				output = output.Replace(TAG_FULLNAME, directoryInfos[i].FullName);
				output = output.Replace(TAG_UPDATETIME, directoryInfos[i].LastWriteTime.ToShortDateString());

				System.TimeSpan timeSpan = DateTime.Today.Subtract(directoryInfos[i].LastWriteTime);
				// if last update to this directory in within 7 days, 
				// replace tag "<updatemark>" to "Updated"
				if (timeSpan.Days <= 7)
				{
					output = output.Replace(TAG_UPDATEMARK, "Updated");
				}
				else
				{
					output = output.Replace(TAG_UPDATEMARK, "");
				}

				output = output.Replace(TAG_ADDONPATH,directoryInfos[i].Name);

				resp.Write(output);

				if (i>=directoryInfos.Length) break;
				output = templateContent;
			}

		}

		#endregion

		#region Translate GalleryInfo according to different template

		/// <summary>
		/// Translate GalleryInfo according to different template, and return the output HTML string
		/// </summary>
		/// <param name="templateFile">The template file with full path</param>
		/// <param name="galleryInfo">The specified GalleryInfo object</param>
		/// <param name="addonpath">The specified addon path</param>
		/// <returns>return translated HTML string</returns>
		public static string Translate(string templateFile, GalleryInfo galleryInfo, string addonpath)
		{
			string retval = "";
			StreamReader streamReader = new StreamReader(templateFile);

			try 
			{
				retval = streamReader.ReadToEnd();
				streamReader.Close();
			}
			catch
			{
				throw new Exception("Cannot read template file " + templateFile);
			}

			if (retval == "") return "";

			// just in case, change the template content to all lower letters
			retval = retval.ToLower();

			retval = retval.Replace(TAG_FILENAME,galleryInfo.FileName);
			retval = retval.Replace(TAG_PIXELFORMAT, galleryInfo.PixelFormat);
			retval = retval.Replace(TAG_FULLNAME, galleryInfo.FullName);
			retval = retval.Replace(TAG_VIRTURALFILENAME, galleryInfo.VirtualFileName);
			retval = retval.Replace(TAG_THUMBNAME, galleryInfo.ThumbName);
			retval = retval.Replace(TAG_SIZE, FormatSize(galleryInfo.Size));
			retval = retval.Replace(TAG_WIDTH, galleryInfo.Width.ToString());
			retval = retval.Replace(TAG_HEIGHT, galleryInfo.Height.ToString());
			retval = retval.Replace(TAG_CLICKCOUNT, galleryInfo.ClickCount.ToString());
			retval = retval.Replace(TAG_CREATETIME, galleryInfo.CreateTime.ToShortDateString());
			retval = retval.Replace(TAG_ADDONPATH, addonpath);

			return retval;
		}

		/// <summary>
		/// Translate GalleryInfo objects according to different template, and return the output HTML string
		/// </summary>
		/// <param name="templateFile">The template file with full path</param>
		/// <param name="galleryInfos">The specified GalleryInfo objects</param>
		/// <param name="addonpath">The specified addon path</param>
		/// <returns>return translated HTML string</returns>
		public static string Translate(string templateFile, GalleryInfo[] galleryInfos, string addonpath)
		{
			string templateContent = "";
			string retval = "";

			StreamReader streamReader = new StreamReader(templateFile);

			try 
			{
				templateContent = streamReader.ReadToEnd();
				streamReader.Close();
			}
			catch
			{
				throw new Exception("Cannot read template file " + templateFile);
			}

			if (templateContent == "") return "";

			// just in case, change the template content to all lower letters
			templateContent = templateContent.ToLower();
			string backupContent = templateContent;

			for (int i=0;i<galleryInfos.Length;i++)
			{
				backupContent = backupContent.Replace(TAG_FILENAME,galleryInfos[i].FileName);
				backupContent = backupContent.Replace(TAG_PIXELFORMAT, galleryInfos[i].PixelFormat);
				backupContent = backupContent.Replace(TAG_FULLNAME, galleryInfos[i].FullName);
				backupContent = backupContent.Replace(TAG_VIRTURALFILENAME, galleryInfos[i].VirtualFileName);
				backupContent = backupContent.Replace(TAG_THUMBNAME, galleryInfos[i].ThumbName);
				backupContent = backupContent.Replace(TAG_SIZE, FormatSize(galleryInfos[i].Size));
				backupContent = backupContent.Replace(TAG_WIDTH, galleryInfos[i].Width.ToString());
				backupContent = backupContent.Replace(TAG_HEIGHT, galleryInfos[i].Height.ToString());
				backupContent = backupContent.Replace(TAG_CLICKCOUNT, galleryInfos[i].ClickCount.ToString());
				backupContent = backupContent.Replace(TAG_CREATETIME, galleryInfos[i].CreateTime.ToShortDateString());
				backupContent = backupContent.Replace(TAG_ADDONPATH,addonpath);

				retval = retval + backupContent;
				if (i>=galleryInfos.Length) break;
				backupContent = templateContent;
			}

			return retval;
		}

		/// <summary>
		/// Translate GalleryInfo objects according to different template, and output the translated HTML string with HttpResponse
		/// </summary>
		/// <param name="templateFile">The template file with full path</param>
		/// <param name="galleryInfo">The specified GalleryInfo object</param>
		/// <param name="addonpath">The specified addon path</param>
		/// <param name="resp">HttpResponse object used to output</param>
		public static void Translate(string templateFile, GalleryInfo galleryInfo, string addonpath, HttpResponse resp)
		{
			string output = "";
			StreamReader streamReader = new StreamReader(templateFile);

			try 
			{
				output = streamReader.ReadToEnd();
				streamReader.Close();
			}
			catch
			{
				throw new Exception("Cannot read template file " + templateFile);
			}

			if (output == "") return;

			// just in case, change the template content to all lower letters
			output = output.ToLower();

			output = output.Replace(TAG_FILENAME,galleryInfo.FileName);
			output = output.Replace(TAG_PIXELFORMAT, galleryInfo.PixelFormat);
			output = output.Replace(TAG_FULLNAME, galleryInfo.FullName);
			output = output.Replace(TAG_VIRTURALFILENAME, galleryInfo.VirtualFileName);
			output = output.Replace(TAG_THUMBNAME, galleryInfo.ThumbName);
			output = output.Replace(TAG_SIZE, FormatSize(galleryInfo.Size));
			output = output.Replace(TAG_WIDTH, galleryInfo.Width.ToString());
			output = output.Replace(TAG_HEIGHT, galleryInfo.Height.ToString());
			output = output.Replace(TAG_CLICKCOUNT, galleryInfo.ClickCount.ToString());
			output = output.Replace(TAG_CREATETIME, galleryInfo.CreateTime.ToShortDateString());
			output = output.Replace(TAG_ADDONPATH,addonpath);

			resp.Write(output);
		}

		/// <summary>
		/// Translate GalleryInfo objects according to different template, and output the translated HTML string with HttpResponse
		/// </summary>
		/// <param name="templateFile">The template file with full path</param>
		/// <param name="galleryInfos">The specified GalleryInfo objects</param>
		/// <param name="addonpath">The specified addon path</param>
		/// <param name="resp">HttpResponse object used to output</param>
		public static void Translate(string templateFile, GalleryInfo[] galleryInfos, string addonpath, HttpResponse resp)
		{
			string templateContent = "";

			StreamReader streamReader = new StreamReader(templateFile);

			try 
			{
				templateContent = streamReader.ReadToEnd();
				streamReader.Close();
			}
			catch
			{
				throw new Exception("Cannot read template file " + templateFile);
			}

			if (templateContent == "") return;

			// just in case, change the template content to all lower letters
			templateContent = templateContent.ToLower();
			string output = templateContent;

			for (int i=0;i<galleryInfos.Length;i++)
			{
				output = output.Replace(TAG_FILENAME,galleryInfos[i].FileName);
				output = output.Replace(TAG_PIXELFORMAT, galleryInfos[i].PixelFormat);
				output = output.Replace(TAG_FULLNAME, galleryInfos[i].FullName);
				output = output.Replace(TAG_VIRTURALFILENAME, galleryInfos[i].VirtualFileName);
				output = output.Replace(TAG_THUMBNAME, galleryInfos[i].ThumbName);
				output = output.Replace(TAG_SIZE, FormatSize(galleryInfos[i].Size));
				output = output.Replace(TAG_WIDTH, galleryInfos[i].Width.ToString());
				output = output.Replace(TAG_HEIGHT, galleryInfos[i].Height.ToString());
				output = output.Replace(TAG_CLICKCOUNT, galleryInfos[i].ClickCount.ToString());
				output = output.Replace(TAG_CREATETIME, galleryInfos[i].CreateTime.ToShortDateString());
				output = output.Replace(TAG_ADDONPATH,addonpath);

				resp.Write(output);

				if (i>=galleryInfos.Length) break;
				output = templateContent;
			}

		}

		#endregion

		static private string FormatSize(long size)
		{
			double s = size/1000.0;
			return  s.ToString("0.0") + " KB";
		}
	}
}
